@@ -6,6 +6,7 @@ import base64
 import json
 import logging
 from dataclasses import dataclass
+from datetime import datetime
 from typing import Any, cast
 
 from aiohttp import ClientSession, ContentTypeError
@@ -16,6 +17,9 @@ from aioqsw.device import (
     FirmwareCheck,
     FirmwareCondition,
     FirmwareInfo,
+    LACPInfo,
+    PortsStatistics,
+    PortsStatus,
     SystemBoard,
     SystemSensor,
     SystemTime,
@@ -24,9 +28,13 @@ from aioqsw.device import (
 from .const import (
     API_AUTHORIZATION,
     API_COMMAND,
+    API_DONE,
+    API_DOWNLOAD_SIZE,
+    API_FIRMWARE_SIZE,
     API_PASSWORD,
     API_PATH,
     API_PATH_V1,
+    API_PROGRESS,
     API_QSW_ID,
     API_QSW_LANG,
     API_REBOOT,
@@ -36,6 +44,8 @@ from .const import (
     QSD_FIRMWARE_CHECK,
     QSD_FIRMWARE_CONDITION,
     QSD_FIRMWARE_INFO,
+    QSD_PORTS_STATISTICS,
+    QSD_PORTS_STATUS,
     QSD_SYSTEM_BOARD,
     QSD_SYSTEM_SENSOR,
     QSD_SYSTEM_TIME,
@@ -79,8 +89,12 @@ class QnapQswApi:
         self.firmware_check: FirmwareCheck | None = None
         self.firmware_condition: FirmwareCondition | None = None
         self.firmware_info: FirmwareInfo | None = None
+        self.firmware_progress: float = 0.0
         self.headers: dict[str, str] = {}
+        self.lacp_info: LACPInfo | None = None
         self.options = options
+        self.ports_statistics: PortsStatistics | None = None
+        self.ports_status: PortsStatus | None = None
         self.system_board: SystemBoard | None = None
         self.system_sensor: SystemSensor | None = None
         self.system_time: SystemTime | None = None
@@ -175,10 +189,22 @@ class QnapQswApi:
         """API GET firmware update check."""
         return await self.http_request("GET", f"{API_PATH_V1}/firmware/update/check")
 
+    async def get_lacp_info(self) -> dict[str, Any]:
+        """API GET LACP info."""
+        return await self.http_request("GET", f"{API_PATH_V1}/lacp/info")
+
     async def get_live(self) -> dict[str, Any]:
         """API GET live."""
         return await self.http_request("GET", f"{API_PATH}/live")
 
+    async def get_ports_statistics(self) -> dict[str, Any]:
+        """API GET ports status."""
+        return await self.http_request("GET", f"{API_PATH_V1}/ports/statistics")
+
+    async def get_ports_status(self) -> dict[str, Any]:
+        """API GET ports status."""
+        return await self.http_request("GET", f"{API_PATH_V1}/ports/status")
+
     async def get_users_verification(self) -> dict[str, Any]:
         """API GET users verification."""
         return await self.http_request("GET", f"{API_PATH_V1}/users/verification")
@@ -206,6 +232,12 @@ class QnapQswApi:
         }
         return await self.http_request("POST", f"{API_PATH_V1}/system/command", params)
 
+    async def post_firmware_update_live(self) -> dict[str, Any]:
+        """API POST live firmware update."""
+        return await self.http_request(
+            "POST", f"{API_PATH_V1}/firmware/update/live", {}
+        )
+
     async def post_users_exit(self) -> dict[str, Any]:
         """API POST users exit."""
         return await self.http_request("POST", f"{API_PATH_V1}/users/exit", {})
@@ -229,21 +261,61 @@ class QnapQswApi:
 
         return await self.get_system_config()
 
+    async def live_update(self) -> bool:
+        """Update QNAP QSW to live firmware."""
+        await self.login()
+
+        self.firmware_progress = 0.0
+        response = await self.post_firmware_update_live()
+
+        result = response.get(API_RESULT, "")
+        if result is not None:
+            raise APIError(f"Error when updating: {response}")
+
+        return True
+
     async def reboot(self) -> bool:
         """Reboot QNAP QSW."""
         await self.login()
 
         response = await self.post_system_command(API_REBOOT)
 
-        result = response.get(API_RESULT)
-        if not result:
+        result = response.get(API_RESULT, "")
+        if result is not None:
             raise APIError(f"Error when rebooting: {response}")
 
         return True
 
+    async def update_progress(self) -> float:
+        """Get QNAP QSW update progress."""
+        response = await self.get_firmware_update()
+
+        result = response.get(API_RESULT)
+        if result and result[API_RESULT] is dict:
+            if result.keys() >= {API_DOWNLOAD_SIZE, API_FIRMWARE_SIZE}:
+                dl_size = float(result[API_DOWNLOAD_SIZE])
+                fw_size = float(result[API_FIRMWARE_SIZE])
+                self.firmware_progress = dl_size * 100.0 / fw_size
+
+        return self.firmware_progress
+
+    async def update_status(self) -> bool:
+        """Get QNAP QSW update status."""
+        response = await self.get_firmware_status()
+
+        result = response.get(API_RESULT, "")
+        if result is None:
+            return False
+        if result and result[API_RESULT] is dict:
+            if API_PROGRESS in result and result[API_PROGRESS] == API_DONE:
+                return False
+
+        return True
+
     async def validate(self) -> SystemBoard:
         """Validate QNAP QSW."""
         self._first_update = True
+        self.ports_statistics = None
 
         try:
             await self.get_live()
@@ -274,11 +346,37 @@ class QnapQswApi:
                 firmware_info = await self.get_firmware_info()
                 self.firmware_info = FirmwareInfo(firmware_info)
 
+            # Update lacp/info once
+            if self.lacp_info is None:
+                lacp_info = await self.get_lacp_info()
+                self.lacp_info = LACPInfo(lacp_info)
+
             # Update system/board once
             if self.system_board is None:
                 system_board = await self.get_system_board()
                 self.system_board = SystemBoard(system_board)
 
+            # Get LACP ports start from lacp/info
+            if self.lacp_info is not None:
+                lacp_start = self.lacp_info.get_start_id()
+            elif self.system_board is not None:
+                lacp_start = self.system_board.get_port_num() + 1
+            else:
+                lacp_start = None
+
+            ports_statistics_data = await self.get_ports_statistics()
+            _datetime = datetime.utcnow()
+            ports_statistics = PortsStatistics(
+                ports_statistics_data, lacp_start, _datetime
+            )
+            ports_statistics.calc(self.ports_statistics)
+            self.ports_statistics = ports_statistics
+
+            ports_status_data = await self.get_ports_status()
+            ports_status = PortsStatus(ports_status_data, lacp_start)
+            ports_status.calc()
+            self.ports_status = ports_status
+
             system_time = await self.get_system_time()
             self.system_time = SystemTime(system_time)
         except QswError as err:
@@ -373,6 +471,16 @@ class QnapQswApi:
             if len(firmware_info_data) > 0:
                 data[QSD_FIRMWARE_INFO] = firmware_info_data
 
+        if self.ports_statistics is not None:
+            ports_statistics_data = self.ports_statistics.data()
+            if len(ports_statistics_data) > 0:
+                data[QSD_PORTS_STATISTICS] = ports_statistics_data
+
+        if self.ports_status is not None:
+            ports_status_data = self.ports_status.data()
+            if len(ports_status_data) > 0:
+                data[QSD_PORTS_STATUS] = ports_status_data
+
         if self.system_board is not None:
             system_board_data = self.system_board.data()
             if len(system_board_data) > 0:
